//
// DummyThreads.hh
//
// Copyright (C) :  2000 - 2002
//					LifeLine Networks BV (www.lifeline.nl). All rights reserved.
//					Bastiaan Bakker. All rights reserved.   
//					
//					2004,2005,2006,2007,2008,2009,2010
//					Synchrotron SOLEIL
//                	L'Orme des Merisiers
//                	Saint-Aubin - BP 48 - France
//
// This file is part of log4tango.
//
// Log4ango is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Log4tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with Log4Tango.  If not, see <http://www.gnu.org/licenses/>.

#ifndef _LOG4TANGO_THREADING_DUMMY_THREADS_H
#define _LOG4TANGO_THREADING_DUMMY_THREADS_H

#include <log4tango/Portability.hh>
#include <stdio.h>
#include <string>

namespace log4tango {

namespace threading {

std::string get_thread_id (void);

typedef int Mutex;

typedef int ScopedLock;

#ifdef LOG4TANGO_HAS_NDC
template<typename T> class ThreadLocalDataHolder 
{
public:
  typedef T data_type;

  inline ThreadLocalDataHolder () 
    : _data(0) 
  {
    //no-op
  };

  inline ~ThreadLocalDataHolder () {
    if (_data) {
      delete _data;
    }
  };
    
  inline T* get (void) const {
    return _data;
  };

  inline T* operator->() const { 
    return get(); 
  };

  inline T& operator*() const { 
    return *get(); 
  };

  inline T* release() {
    T* result = _data;
    _data = 0;
    return result;
  };

  inline void reset (T* p = 0) {
    if (_data) { 
      delete _data;
    }
    _data = p;
  };

private:            
    T* _data;            
};
#endif // #ifdef LOG4TANGO_HAS_NDC

} // namespace threading

} // namespace log4tango

#endif // _LOG4TANGO_THREADING_DUMMY_THREADS_H
